@@ -6,7 +6,7 @@
  * PHP object-oriented interface library for Yate
  *
  * Yet Another Telephony Engine - a fully featured software PBX and IVR
- * Copyright (C) 2004-2014 Null Team
+ * Copyright (C) 2004-2023 Null Team
  *
  * This software is distributed under multiple licenses;
  * see the COPYING file in the main directory for licensing
@@ -49,17 +49,26 @@ class Yate
 
     /**
      * Static function to output a string to Yate's stderr or logfile
+     * NOTE: Yate expects an escaped string for 'debug'
      * @param $str String to output
+     * @param $level Debug level. Send a debug message if greater than 0
+     * @param $escape Escape the string to output
      */
-    static function Output($str)
+    static function Output($str,$level = 0,$escape = false)
     {
 	global $yate_stderr, $yate_output;
 	if ($str === true)
 	    $yate_output = true;
 	else if ($str === false)
 	    $yate_output = false;
-	else if ($yate_output)
-	    _yate_print("%%>output:$str\n");
+	else if ($yate_output) {
+	    if ($escape)
+		$str = Yate::Escape($str);
+	    if ($level <= 0)
+		_yate_print("%%>output:$str\n");
+	    else
+		_yate_print("%%>debug:$level:$str\n");
+	}
 	else
 	    fputs($yate_stderr, "$str\n");
     }
@@ -68,8 +77,10 @@ class Yate
      * Static function to output a string to Yate's stderr or logfile
      *  only if debugging was enabled.
      * @param $str String to output, or boolean (true/false) to set debugging
+     * @param $level Debug level. Send a debug message if greater than 0
+     * @param $escape Escape the string to output
      */
-    static function Debug($str)
+    static function Debug($str,$level = 0,$escape = false)
     {
 	global $yate_stderr, $yate_debug;
 	if ($str === true)
@@ -77,7 +88,7 @@ class Yate
 	else if ($str === false)
 	    $yate_debug = false;
 	else if ($yate_debug)
-	    Yate::Output($str);
+	    Yate::Output($str,$level,$escape);
     }
 
     /**
@@ -127,7 +138,7 @@ class Yate
 	$s = "";
 	$n = strlen($str);
 	for ($i=0; $i<$n; $i++) {
-	    $c = $str{$i};
+	    $c = $str[$i];
 	    if ((ord($c) < 32) || ($c == ':') || ($c == $extra)) {
 		$c = chr(ord($c) + 64);
 		$s .= '%';
@@ -149,10 +160,10 @@ class Yate
 	$s = "";
 	$n = strlen($str);
 	for ($i=0; $i<$n; $i++) {
-	    $c = $str{$i};
+	    $c = $str[$i];
 	    if ($c == '%') {
 		$i++;
-		$c = $str{$i};
+		$c = $str[$i];
 		if ($c != '%')
 		    $c = chr(ord($c) - 64);
 	    }
@@ -235,7 +246,7 @@ class Yate
      * @param $retval (optional) Default return
      * @param $id (optional) Identifier of the new message
      */
-    function Yate($name, $retval = "", $id = "")
+    function __construct($name, $retval = "", $id = "")
     {
 	if ($id == "")
 	    $id=uniqid(rand(),1);
@@ -302,12 +313,10 @@ class Yate
 	    return;
 	}
 	$i=Yate::Escape($this->id);
-	$t=0+$this->origin;
+	$t=(int)$this->origin;
 	$n=Yate::Escape($this->name);
 	$r=Yate::Escape($this->retval);
-	$p="";
-	$pa = array(&$p);
-	array_walk($this->params, "_yate_message_walk", $pa);
+	$p = _yate_escaped_params($this->params);
 	_yate_print("%%>message:$i:$t:$n:$r$p\n");
 	$this->type="dispatched";
     }
@@ -326,9 +335,7 @@ class Yate
 	$k=Yate::Bool2str($this->handled);
 	$n=Yate::Escape($this->name);
 	$r=Yate::Escape($this->retval);
-	$p="";
-	$pa = array(&$p);
-	array_walk($this->params, "_yate_message_walk", $pa);
+	$p = _yate_escaped_params($this->params);
 	_yate_print("%%<message:$i:$k:$n:$r$p\n");
 	$this->type="acknowledged";
     }
@@ -379,7 +386,7 @@ class Yate
 		/* incoming message str_id:int_time:str_name:str_retval[:key=value...] */
 		$ev=new Yate(Yate::Unescape($part[3]),Yate::Unescape($part[4]),Yate::Unescape($part[1]));
 		$ev->type="incoming";
-		$ev->origin=0+$part[2];
+		$ev->origin=(int)$part[2];
 		$ev->FillParams($part,5);
 		break;
 	    case "%%<message":
@@ -391,13 +398,13 @@ class Yate
 		break;
 	    case "%%<install":
 		/* install answer num_priority:str_name:bool_success */
-		$ev=new Yate(Yate::Unescape($part[2]),"",0+$part[1]);
+		$ev=new Yate(Yate::Unescape($part[2]),"",(int)$part[1]);
 		$ev->type="installed";
 		$ev->handled=Yate::Str2bool($part[3]);
 		break;
 	    case "%%<uninstall":
 		/* uninstall answer num_priority:str_name:bool_success */
-		$ev=new Yate(Yate::Unescape($part[2]),"",0+$part[1]);
+		$ev=new Yate(Yate::Unescape($part[2]),"",(int)$part[1]);
 		$ev->type="uninstalled";
 		$ev->handled=Yate::Str2bool($part[3]);
 		break;
@@ -588,6 +595,15 @@ function _yate_print($str)
     }
 }
 
+// Internal function: escape and concatenate a list of parameters
+function _yate_escaped_params($params)
+{
+    $ep = "";
+    foreach ($params as $key=>$item)
+	$ep .= ':' . Yate::Escape($key,'=') . '=' . Yate::Escape($item);
+    return $ep;
+}
+
 /* Internal function */
 function _yate_message_walk($item, $key, &$result)
 {
